Imports System.Text
Imports System.IO

'Durch Verwendung des Interfaces erwirke ich, da die Intellisense bei Verwendung dieser Methoden nur wesentliche Member anbietet

Public Interface ILogger
    Function Add(ByVal ParamArray Segments() As Object) As ILogger
    Sub OpenLogFile()
End Interface

Public Class Logger : Implements ILogger

    Private _LogFile As String
    Private Shared ReadOnly _Singleton As New Logger
    Private WithEvents _AppDomain As AppDomain

    Private _Writer As StreamWriter

    Private Sub New()
        Dim Segments As String() = { _
            Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), _
            Application.CompanyName, _
            Application.ProductName, _
            "Errors", _
            "Error.txt"}
        _LogFile = String.Join(Path.DirectorySeparatorChar, Segments)
        Dim sDir As String = Path.GetDirectoryName(_LogFile)
        If Not Directory.Exists(sDir) Then Directory.CreateDirectory(sDir)
        _Writer = New StreamWriter(_LogFile, Append:=False, Encoding:=Encoding.UTF8)
        _AppDomain = AppDomain.CurrentDomain
    End Sub

    Public Shared Function Entry(ByVal ParamArray Segments() As Object) As ILogger
        Return _Singleton.Entry1(Segments)
    End Function

    Public Shared Sub OpenLogFile()
        _Singleton.OpenLogFile1()
    End Sub

    Private Function Entry1(ByVal ParamArray Segments() As Object) As ILogger
        ' Die Eintrge knnen auch wesentlich aufwndiger formatiert werden (z.B. Xml generieren).
        ' z.B. knnte hier auch ein Timestamp generiert werden. 
        _Writer.WriteLine()
        Add(Segments)
        Return Me
    End Function

    Private Function Add(ByVal ParamArray Segments() As Object) As ILogger _
            Implements ILogger.Add
        For Each Obj As Object In Segments
            If Obj Is Nothing Then
                _Writer.Write("#Null#")
            Else
                _Writer.Write(Obj.ToString)
            End If
        Next
        _Writer.WriteLine()
        ' Da Add() den Singleton selbst zurckgibt, kann man mehrere Aufrufe aneinanderketten.
        ' Das schreibt sich einfacher als ein With-Block und ist z.T. sogar besser lesbar.
        Return Me
    End Function

    Private Sub OpenLogFile1() Implements ILogger.OpenLogFile
        _Writer.Flush() 'akt. Puffer wegschreiben
        ' LogFile mit Extension-spezifischer Anwendung ffnen
        System.Diagnostics.Process.Start(_LogFile)
    End Sub

    Private Sub AppDomain_ProcessExit(ByVal sender As Object, ByVal e As System.EventArgs) _
            Handles _AppDomain.ProcessExit
        _Writer.Dispose()
    End Sub

    Private Sub AppDomain_UnhandledException( _
            ByVal sender As Object, ByVal e As UnhandledExceptionEventArgs) _
            Handles _AppDomain.UnhandledException
        If e.IsTerminating Then _Writer.Dispose()
    End Sub

End Class
